using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixCards.Class;
using PhoenixDLL.Class.Managers;
using Utils.Logs;
using System.Collections;
using PhoenixDLL.Class;
using Components.Calendar;
using Utils.DataBase;

namespace PhoenixCards.GUI
{
    /// <summary>
    /// Frame to manage the meetings
    /// </summary>
    public partial class fr_Today : Form
    {
        #region Variables
        /// <summary>
        /// Manager for the payment
        /// </summary>
        private TimeCardManager vTimeCardManager;
        /// <summary>
        /// Selected Meeting
        /// </summary>
        private MeetingManager vSelectedMeeting;
        /// <summary>
        /// Contains the meetings to dispay in the Calendar
        /// </summary>
        private MeetingSet vMeetingSet;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public fr_Today() { InitializeComponent(); }
        #endregion

        #region Events
        /// <summary>
        /// The frame is loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e)
        {
            MeetingSelected();

            tssl_Patient.Text = vTimeCardManager.PatientManager.Surname
                + " " + vTimeCardManager.PatientManager.FamillyName.ToUpper();
            tssl_Debt.Text = "";
        }
        /// <summary>
        /// Event on presence selection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_PresentNo_CheckedChanged(object sender, EventArgs e)
        {
            btn_DelMeeting.Enabled = true;
            gb_Payed.Enabled = false;
            btn_Pay.Enabled = false;
            btn_NotPay.Text = "Quitter";
            btn_NotPay.Enabled = true;
            btn_Modify.Enabled = false;
        }
        /// <summary>
        /// Event on presence selection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_PresentYes_CheckedChanged(object sender, EventArgs e)
        {
            int Pay;
            btn_DelMeeting.Enabled = false;
            gb_Payed.Enabled = true;
            btn_NotPay.Text = "Ne pas payer";
            if (!Int32.TryParse(tb_Payment.Text, out Pay))
            {
                btn_Pay.Enabled = false;
                tb_Payment.Text = "";
            }
            else
                btn_Pay.Enabled = true;
            btn_Modify.Enabled = true;
        }
        /// <summary>
        /// Event in click for emeritus modification
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Modify_Emeritus(object sender, EventArgs e)
        {
            if (tb_Honoraire.ReadOnly == true)
            {
                btn_NotPay.Enabled = false;
                tb_Honoraire.ReadOnly = false;
                btn_Pay.Enabled = false;
                btn_Modify.Text = "Appliquer";
            }
            else
            {
                int Res;
                if (!Int32.TryParse(tb_Honoraire.Text, out Res))
                {
                    MessageBox.Show("Veuillez entrer une valeur numrique pour le prix de l'honoraire!",
                        "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    CalculateDebt();
                    btn_Modify.Text = "Modifier";
                    //btn_Pay.Enabled = rb_PresentYes.Checked ? true : false;
                    tb_Honoraire.ReadOnly = true;
                    btn_NotPay.Enabled = true;
                }
            }
        }
        /// <summary>
        /// Event on pay asking
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Pay(object sender, EventArgs e)
        {
            int Pay;
            int Hon;
            Int32.TryParse(tb_Honoraire.Text, out Hon);
            Int32.TryParse(tb_Payment.Text, out Pay);

            if (Hon - Pay < 0)
            {
                MessageBox.Show("Le patient paye plus qu'il ne doit, veuillez entrer une somme infrieure ou "
                    + "gale aux honoraires!", "Attention", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                DialogResult dr = MessageBox.Show("Voulez-vous effectuer le payement?",
                    "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.No) return;

                //Payement
                vSelectedMeeting.Came = true;
                if (Hon - Pay == 0)
                    vSelectedMeeting.Payed = true;
                else
                {
                    vSelectedMeeting.Payed = false;
                    vTimeCardManager.PatientManager.AddToDebt(Hon - Pay);
                    vTimeCardManager.PatientManager.Update();
                }

            }
            vSelectedMeeting.Update();
            //If the debt == 0 all the camed rdv are marked as payed
            if (vTimeCardManager.PatientManager.Debt == 0)
            {
                OleDbSql sm = new OleDbSql(Options.Instance.Path);
                sm.SqlQuery = "UPDATE RendezVous SET a_paye = TRUE, came_to_rdv = TRUE WHERE id_patient = "
                    + this.vTimeCardManager.PatientManager.IDPatient + ";";
                sm.ExecuteNonQuery();
            }

            this.Close();
            base.DialogResult = DialogResult.OK;
        }
        /// <summary>
        /// Delete the selected meeting
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Del(object sender, EventArgs e)
        {
            try
            {
                DialogResult dr = MessageBox.Show("Voulez-vous supprimer ce rendez-vous? ("
                    + vSelectedMeeting.ToString() + ")?", "Question",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.No) return;
                //this.vSelectedMeeting.Delete();
                this.vTimeCardManager.Meetings.Remove(vSelectedMeeting);
                vMeetingSet.Remove(vSelectedMeeting.IDMeeting);

            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                MessageBox.Show("Une erreur est intervenue lors de la suppression!",
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally { this.Close(); }
        }
        /// <summary>
        /// click on not pay the meeting
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_NotPay(object sender, EventArgs e)
        {
            if (btn_NotPay.Text == "Quitter")
            {
                this.Close();
                return;
            }

            int Sum = 0;
            bool Parsed = Int32.TryParse(tb_Honoraire.Text, out Sum);
            if (!Parsed)
            {
                MessageBox.Show("Les honoraires sont illisible!", "Erreur",
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }
            DialogResult dr = MessageBox.Show("Le patient n'a pas pay. Ajout de " + Sum
              + Options.Instance.MoneySign + "  la dette. Continuer?", "Question",
              MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.No) return;

            //Mark the rdv as non came
            OleDbSql sm = new OleDbSql(Options.Instance.Path);
            sm.SqlQuery = "UPDATE RendezVous SET a_paye = TRUE,came_to_rdv = FALSE WHERE"
                + " id_patient = " + this.vTimeCardManager.PatientManager.IDPatient + ";";
            vTimeCardManager.PatientManager.AddDebt(Sum);
            vTimeCardManager.PatientManager.Update();
            sm.ExecuteNonQuery();

            // Modify the debt

            base.DialogResult = DialogResult.OK;
            this.Close();
        }
        /// <summary>
        /// Event on selection of a meeting in the list
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Meetings_SelectedIndexChanged(object sender, EventArgs e)
        {
            vSelectedMeeting = (MeetingManager)lb_Meetings.Items[lb_Meetings.SelectedIndex];
            if (vSelectedMeeting != null)
            {
                lbl_Selected.Text = "Slectionn:\r\n" + vSelectedMeeting.ToString();
            }
            else
            {
                btn_Pay.Enabled = false;
                lbl_Selected.Text = "";
            }
        }
        /// <summary>
        /// Text of the tb_Payment changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_Payment_TextChanged(object sender, EventArgs e)
        {
            CalculateDebt();
        }
        #endregion

        #region Methods
        /// <summary>
        /// Loads default setting on meeting selection
        /// </summary>
        private void MeetingSelected()
        {
            lb_Meetings.DataSource = this.vTimeCardManager.Meetings;
            tb_Honoraire.Text = this.vTimeCardManager.PatientManager.Honoraire.ToString();
            btn_Pay.Enabled = false;
        }
        /// <summary>
        /// Calculates the debt
        /// </summary>
        private void CalculateDebt()
        {
            int Hon = 0;
            Int32.TryParse(tb_Honoraire.Text, out Hon);
            int Pay;
            if (Int32.TryParse(tb_Payment.Text, out Pay))
            {
                btn_Pay.Enabled = true;

                tssl_Debt.Text = "Dette: " + (Hon - Pay) + " " + Options.Instance.MoneySign;
                if (Hon - Pay >= 0)
                    tssl_Debt.BackColor = Color.Green;
                else
                    tssl_Debt.BackColor = Color.Red;
            }
            else
            {
                btn_Pay.Enabled = false;
                tssl_Debt.Text = "";
            }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the TimeCardManager
        /// </summary>
        public TimeCardManager TimeCardManager
        {
            get { return vTimeCardManager; }
            set { vTimeCardManager = value; }
        }
        /// <summary>
        /// Gets or sets the meetings list to display in the calendar
        /// </summary>
        public MeetingSet MeetingSet
        {
            get { return vMeetingSet; }
            set { vMeetingSet = value; }
        }
        #endregion
    }
}