using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.OleDb;
using System.Threading;
using System.IO;
using System.Drawing.Imaging;
using Utils.DataBase;
using PhoenixDLL.Class;

namespace PhoenixDBFiller
{

    /// <summary>
    /// Fentre principale
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Version of the source database
        /// </summary>
        private Version vVersion = Version.Sharpenda;
        /// <summary>
        /// Path de la BD1
        /// </summary>
        private string vPathDB1 = "";
        /// <summary>
        /// Path de la BD2
        /// </summary>
        private string vPathDB2 = "";
        private string vPathPhotos = "";
        /// <summary>
        /// Thread
        /// </summary>
        private Thread vThread;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();

            DateTime dt = DateTime.Now;
        }
        #endregion

        #region Events
        /// <summary>
        /// Clique sur choix BD1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_DB1(object sender, EventArgs e)
        {
            DialogResult dr = ofd_DB1.ShowDialog();
            if (dr == DialogResult.OK)
            {
                vPathDB1 = ofd_DB1.FileName;
                tb_Log.Text += "Chemin de la Source : " + vPathDB1 + "\r\n";
            }

        }
        /// <summary>
        /// Clique sur choix BD2
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_DB2(object sender, EventArgs e)
        {
            DialogResult dr = ofd_DB2.ShowDialog();
            if (dr == DialogResult.OK)
            {
                vPathDB2 = ofd_DB2.FileName;
                tb_Log.Text += "Chemin de la Cible : " + vPathDB2 + "\r\n";
            }
        }
        /// <summary>
        /// Choix du path pour les photos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_ChoixPhotos(object sender, EventArgs e)
        {
            DialogResult dr = fbd_Photos.ShowDialog();
            if (dr == DialogResult.OK)
            {
                vPathPhotos = fbd_Photos.SelectedPath;
                tb_Log.Text += "Chemin de base pour les photos: \"" + vPathPhotos + "\"";
            }
        }
        /// <summary>
        /// Ckique sur Excuter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Process(object sender, EventArgs e)
        {
            #region Vrification des conditions d'excution
            if (vPathDB1 == "" || vPathDB2 == "")//|| vPathPhotos == "")
            {
                MessageBox.Show("Choisissez les bases de donnes dans les menus!",
                    "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                tb_Log.Text += "Toutes les BD tous les chemins ne sont pas selectionnes!\r\n";
            }
            #endregion

            else
            {
                tb_Log.Text += "\r\n===================================================================\r\n";
                tb_Log.Text += "Version: " + vVersion.ToString() + "\r\n";
                tb_Log.Text += "Excution du transfert...\r\n\r\n";

                #region Processing
                IThreadProcessor th;
                switch (vVersion)
                {
                    case Version.Sharpenda:
                        th = new ThreadProcessorSharpenda();
                        break;
                    case Version.V0913v1:
                        th = new ThreadProcessorV0913v1();
                        break;
                    default:
                        th = new ThreadProcessorSharpenda();
                        break;
                }
                th.GUI = this;
                th.PathDB1 = vPathDB1;
                th.PathDB2 = vPathDB2;
                vThread = new Thread(new ThreadStart(th.Process));
                vThread.Start();
                #endregion
            }
        }
        /// <summary>
        /// On quitte la fentre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (vThread != null)
                vThread.Abort();
        }
        /// <summary>
        /// Changement du texte dans la TextBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_TextChanged(object sender, EventArgs e)
        {
            tb_Log.Focus();
            tb_Log.SelectionStart = tb_Log.Text.Length;
            tb_Log.SelectionLength = 0;
            tb_Log.ScrollToCaret();
        }
        /// <summary>
        /// Event when the user select a version of the source database
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_VersionSelector(object sender, EventArgs e)
        {
            #region Uncheck the versions selectors
            this.tsMenu_0913V1.Checked = false;
            this.tsMenu_Sharpenda.Checked = false;
            #endregion

            if (sender == this.tsMenu_Sharpenda)
            {
                this.tsMenu_Sharpenda.Checked = true;
                vVersion = Version.Sharpenda;
            }
            else if (sender == this.tsMenu_0913V1)
            {
                this.tsMenu_0913V1.Checked = true;
                vVersion = Version.V0913v1;
            }

        }
        #endregion
    }
    internal enum Version
    {
        /// <summary>
        /// Sharpenda's database
        /// </summary>
        Sharpenda,
        /// <summary>
        /// V0913v1's database
        /// </summary>
        V0913v1,
    }
    /// <summary>
    /// Interface for the implementation of the Thread treatment of a db filler
    /// </summary>
    internal interface IThreadProcessor
    {
        /// <summary>
        /// Process de transfert d'une base de donnes vers une autre
        /// </summary>
        void Process();
        /// <summary>
        /// Emplacement de la Base de Donnes 1
        /// </summary>
        string PathDB1 { get; set; }
        /// <summary>
        /// Emplacement de la Base de Donnes 2
        /// </summary>
        string PathDB2 { get; set; }
        /// <summary>
        /// Retourne ou initialise la frame de la GUI
        /// </summary>
        fr_Main GUI { get; set; }
    }
    /// <summary>
    /// Thread for the version Sharpenda database
    /// </summary>
    internal class ThreadProcessorSharpenda : IThreadProcessor
    {
        #region variables
        /// <summary>
        /// Emplacement de la BD
        /// </summary>
        private string vPathBdSource = "";
        /// <summary>
        /// Path de la deuxime base de donnes
        /// </summary>
        private string vPathBdTarget = "";
        /// <summary>
        /// Frame de la GUI
        /// </summary>
        private fr_Main frame;
        /// <summary>
        /// Sql Manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);

        #region PerformStep
        /// <summary>
        /// Delegate pour l'eccutio d'un pas  la ProgressBar
        /// </summary>
        private delegate void GuiPerformStepHandler();
        /// <summary>
        /// Mthode de traitement pour l'excution d'un pas a la ProgressBar
        /// </summary>
        private GuiPerformStepHandler PerformStep;
        #endregion

        #region Log
        /// <summary>
        /// Delegate pour le loggage dans la TextBox
        /// </summary>
        /// <param name="_ToLog">Texte  logger</param>
        private delegate void GuiLogHandler(string _ToLog);

        /// <summary>
        /// Mthode pour le loggage dans le Text Box
        /// </summary>
        private GuiLogHandler Log;
        #endregion

        #region Setup ProgressBar
        /// <summary>
        /// Dekegate pour le setup de la ProgressBar
        /// </summary>
        /// <param name="_Maximum">Maximum de la ProgresBar</param>
        /// <param name="_Minimum">Minimum de la ProgresBar</param>
        /// <param name="_Step">Nombre d'tapes  faire lors d'un pas</param>
        /// <param name="_Value">Valeur de base du pas de la ProgressBAr</param>
        private delegate void GuiSetupProgressBarHandler(int _Maximum, int _Minimum, int _Step, int _Value);

        /// <summary>
        /// Mthode pour le setup de la ProgressBar
        /// </summary>
        private GuiSetupProgressBarHandler SetupProgressBar;
        #endregion
        #endregion

        #region Constructeur
        #endregion

        #region Mthodes
        /// <summary>
        /// Process de transfert d'une base de donnes vers une autre
        /// </summary>
        public void Process()
        {
            try
            {

                #region Selection des types de mdicaments
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des types de mdicaments... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                DataSet ds = new DataSet();
                SqlManager.DbPath = vPathBdSource;
                int rows = SqlManager.ExecuteSql("SELECT * FROM Type_Medicament;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des types de mdicaments dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des mdicaments dans la nouvelle base de donnes...");

                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int ID_Type = (int)dr["ID_Type"];

                    int id_Type = -1;
                    if (!dr["Type"].Equals(DBNull.Value))
                        id_Type = (int)dr["id_Type"];

                    string Type = "";
                    if (!dr["Prix"].Equals(DBNull.Value))
                        Type = (string)dr["Prix"];

                    string Nom = "";
                    if (!dr["Nom"].Equals(DBNull.Value))
                        Nom = (string)dr["Nom"];
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Medicament (ID_Type, Type, Rmq, Nom) VALUES("
                        + ID_Type + ",\"" + Type + "\",\"" + Nom + "\");";
                    #endregion

                    #region Execute Query

                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des Mdicaments
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des mdicaments... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Medicament;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des mdicaments dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des mdicaments dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Medicament = (int)dr["id_Honoraire"];

                    int id_Type = -1;
                    if (!dr["Type"].Equals(DBNull.Value))
                        id_Type = (int)dr["id_Type"];

                    short Rmq = 0;
                    if (!dr["Prix"].Equals(DBNull.Value))
                        Rmq = (short)dr["Prix"];

                    string Nom = "";
                    if (!dr["Nom"].Equals(DBNull.Value))
                        Nom = (string)dr["Nom"];
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Medicament (id_Medicament, id_Type, Rmq, Nom) VALUES("
                        + id_Medicament + "," + id_Type + ",\"" + Rmq + "\",\"" + Nom + "\");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des rputations
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des rputations... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Reputation;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des rputations dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des rputations dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Reputation = (int)dr["id_Reputation"];

                    string Label = "";
                    if (!dr["Label"].Equals(DBNull.Value))
                        Label = (string)dr["Label"];

                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Reputation (id_Reputation, Label) VALUES("
                        + id_Reputation + ",\"" + Label + "\");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des honoraires

                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des honoraires... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Honoraire;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des honoraires dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des honoraires dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Honoraire = (int)dr["id_Honoraire"];

                    string Type = "";
                    if (!dr["Type"].Equals(DBNull.Value))
                        Type = (string)dr["Type"];

                    short Prix = 0;
                    if (!dr["Prix"].Equals(DBNull.Value))
                        Prix = (short)dr["Prix"];
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Honoraire (id_Honoraire, Type, Prix) VALUES("
                        + id_Honoraire + ",\"" + Type + "\"," + Prix + ");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des mdecins
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des mdecins... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Medecin;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des mdecins dans la nouvelle DB

                #region Create and process Sql Query
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des mdecins dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_medecin = (int)dr["id_medecin"];

                    string Nom = " ";
                    if (!dr["Nom"].Equals(DBNull.Value))
                        Nom = (string)dr["Nom"];

                    string Prenom = " ";
                    if (!dr["Prenom"].Equals(DBNull.Value))
                        Prenom = (string)dr["Prenom"];

                    string Adresse = " ";
                    if (!dr["Adresse"].Equals(DBNull.Value))
                        Adresse = (string)dr["Adresse"];

                    string Ville = " ";
                    if (!dr["Ville"].Equals(DBNull.Value))
                        Ville = (string)dr["Ville"];

                    string CodePostal = " ";
                    if (!dr["CodePostal"].Equals(DBNull.Value))
                        CodePostal = (string)dr["CodePostal"];

                    string Telephone = " ";
                    if (!dr["Telephone"].Equals(DBNull.Value))
                        Telephone = (string)dr["Telephone"];

                    string Fax = " ";
                    if (!dr["Fax"].Equals(DBNull.Value))
                        Fax = (string)dr["Fax"];

                    bool Homeopathe = true;
                    if (!dr["Homeopathe"].Equals(DBNull.Value))
                        Homeopathe = (bool)dr["Homeopathe"];

                    bool MedecinTraitant = true;
                    if (!dr["MedecinTraitant"].Equals(DBNull.Value))
                        MedecinTraitant = (bool)dr["MedecinTraitant"];
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Medecin (id_medecin, Nom, Prenom, Adresse, Ville, CodePostal, "
                        + "Telephone, Fax, Homeopathe, MedecinTraitant) VALUES(" + id_medecin + ",\"" + Nom + "\",\""
                        + Prenom + "\",\"" + Adresse + "\",\"" + Ville + "\",\"" + CodePostal + "\",\"" + Telephone
                        + "\",\"" + Fax + "\"," + Homeopathe + "," + MedecinTraitant + ")";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #endregion

                #region Selection des patients
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des patients... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM PATIENT;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des patient dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des patients dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Patient = (int)dr["id_Patient"];

                    string Nom = "";
                    if (!dr["Nom"].Equals(DBNull.Value))
                        Nom = (string)dr["Nom"];

                    string Prenom = "";
                    if (!dr["Prenom"].Equals(DBNull.Value))
                        Prenom = (string)dr["Prenom"];

                    string LieuNaissance = "";
                    if (!dr["Lieu_Naissance"].Equals(DBNull.Value))
                        LieuNaissance = (string)dr["Lieu_Naissance"];

                    DateTime DateNaissance = new DateTime();
                    if (!dr["DateNaissance"].Equals(DBNull.Value))
                        DateNaissance = (DateTime)dr["DateNaissance"];

                    string Adresse = "";
                    if (!dr["Adresse"].Equals(DBNull.Value))
                        Adresse = (string)dr["Adresse"];

                    string Code_Postal = "";
                    if (!dr["Code_Postal"].Equals(DBNull.Value))
                        Code_Postal = (string)dr["Code_Postal"];


                    string Profession = "";
                    if (!dr["Profession"].Equals(DBNull.Value))
                        Profession = (string)dr["Profession"];

                    string Ville = "";
                    if (!dr["Ville"].Equals(DBNull.Value))
                        Ville = (string)dr["Ville"];

                    string TelPriv = "";
                    if (!dr["Telephone_Prive"].Equals(DBNull.Value))
                        TelPriv = (string)dr["Telephone_Prive"];

                    string TelPro = "";
                    if (!dr["Telephone_Professionel"].Equals(DBNull.Value))
                        TelPro = (string)dr["Telephone_Professionel"];

                    string Sexe = "";
                    if (!dr["Sexe"].Equals(DBNull.Value))
                        Sexe = (string)dr["Sexe"];

                    string Mail = "";
                    if (!dr["Mail"].Equals(DBNull.Value))
                        Mail = (string)dr["Mail"];

                    int idr = -1;
                    if (!dr["id_Reputation"].Equals(DBNull.Value))
                        idr = (int)dr["id_Reputation"];

                    int idh = -1;
                    if (!dr["id_Homeopathe"].Equals(DBNull.Value))
                        idh = (int)dr["id_Homeopathe"];

                    int idm = -1;
                    if (!dr["id_Medecin"].Equals(DBNull.Value))
                        idm = (int)dr["id_Medecin"];

                    int idho = -1;
                    try
                    {

                        if (!dr["id_honoraire"].Equals(DBNull.Value))
                            idho = (int)dr["id_honoraire"];
                    }
                    catch (Exception) { idho = -1; }

                    string Motif1 = "";
                    if (!dr["Motif_1"].Equals(DBNull.Value))
                        Motif1 = (string)dr["Motif_1"];

                    string Motif2 = "";
                    if (!dr["Motif_2"].Equals(DBNull.Value))
                        Motif2 = (string)dr["Motif_2"];

                    DateTime Derniere_MAJ = new DateTime();
                    if (!dr["Derniere_MAJ"].Equals(DBNull.Value))
                        Derniere_MAJ = (DateTime)dr["Derniere_MAJ"];

                    string FicheMedicale = "";
                    if (!dr["FicheMedicale"].Equals(DBNull.Value))
                        FicheMedicale = (string)dr["FicheMedicale"];

                    string Biodynamics = "";
                    if (!dr["Biodynamics"].Equals(DBNull.Value))
                        Biodynamics = (string)dr["Biodynamics"];

                    string HE = "";
                    if (!dr["HE"].Equals(DBNull.Value))
                        HE = (string)dr["HE"];

                    string Kinergetique = "";
                    if (!dr["Kinergetique"].Equals(DBNull.Value))
                        Kinergetique = (string)dr["Kinergetique"];

                    string FICHE_SUPPL_1 = "";
                    if (!dr["FICHE_SUPPL_1"].Equals(DBNull.Value))
                        FICHE_SUPPL_1 = (string)dr["FICHE_SUPPL_1"];

                    string FICHE_SUPPL_2 = "";
                    if (!dr["FICHE_SUPPL_2"].Equals(DBNull.Value))
                        FICHE_SUPPL_2 = (string)dr["FICHE_SUPPL_2"];

                    string FICHE_SUPPL_3 = "";
                    if (!dr["FICHE_SUPPL_3"].Equals(DBNull.Value))
                        FICHE_SUPPL_3 = (string)dr["FICHE_SUPPL_3"];

                    string FICHE_SUPPL_4 = "";
                    if (!dr["FICHE_SUPPL_4"].Equals(DBNull.Value))
                        FICHE_SUPPL_4 = (string)dr["FICHE_SUPPL_4"];

                    string rdvRemarques = "";
                    if (!dr["rdvRemarques"].Equals(DBNull.Value))
                        rdvRemarques = (string)dr["rdvRemarques"];

                    bool Mauvais_Payeur = false;
                    if (!dr["Mauvais_Payeur"].Equals(DBNull.Value))
                        Mauvais_Payeur = (bool)dr["Mauvais_Payeur"];

                    DateTime Prem_Consult = DateTime.Today;
                    if (!dr["Prem_Consult"].Equals(DBNull.Value))
                        Prem_Consult = (DateTime)dr["Prem_Consult"];

                    int Dette_Patient = 0;
                    try
                    {
                        if (!dr["Dette_Patient"].Equals(DBNull.Value))
                            Dette_Patient = (int)dr["Dette_Patient"];
                    }
                    catch (Exception) { Dette_Patient = 0; }


                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Patient ( id_Patient,Nom, Prenom, Lieu_Naissance, DateNaissance,"
                        + " Adresse, Code_Postal, Profession, Ville, Telephone_Prive, Telephone_Professionel, Sexe, Mail, "
                        + "id_Reputation, id_Medecin,id_Homeopathe,id_honoraire,  Motif_1, Motif_2, Derniere_MAJ, "
                        + "FicheMedicale, Biodynamics, HE, Kinergetique, FICHE_SUPPL_1, FICHE_SUPPL_2, FICHE_SUPPL_3,"
                        + " FICHE_SUPPL_4, rdvRemarques, Mauvais_Payeur, Dette_Patient,Prem_Consult, Honoraire) "
                        + "VALUES (" + id_Patient + ",\"" + Nom + "\", \"" + Prenom + "\", \"" + LieuNaissance
                        + "\", FORMAT(#" + DateNaissance.ToShortDateString() + "#,\"dd/mm/yyyy\"), \"" + Adresse
                        + "\", \"" + Code_Postal + "\", \"" + Profession + "\", \"" + Ville + "\", \"" + TelPriv
                        + "\", \"" + TelPro + "\", \"" + Sexe + "\", \"" + Mail + "\", ";

                    if (idr == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idr + ",";

                    if (idh == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idh + ",";

                    if (idm == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idm + ",";

                    if (idho == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idho + ",";

                    SqlQuery += "\"" + Motif1 + "\", \"" + Motif2
                    + "\", FORMAT(#" + Derniere_MAJ.ToShortDateString() + "#,\"dd/mm/yyyy\"), \"" + FicheMedicale + "\",\""
                    + Biodynamics + "\",\"" + HE + "\",\"" + Kinergetique + "\",\"" + FICHE_SUPPL_1 + "\",\""
                    + FICHE_SUPPL_2 + "\",\"" + FICHE_SUPPL_3 + "\",\"" + FICHE_SUPPL_4 + "\",\"" + rdvRemarques
                    + "\"," + Mauvais_Payeur + "," + Dette_Patient + ",FORMAT(#" + Prem_Consult + "#,\"dd/mm/yyyy\"),0);";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des rendez-vous
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des rendez-vous... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM RendezVous;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des rendez-vous dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des rendez-vous dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {

                    #region Variables
                    int id_Rdv = (int)dr["id_RendezVous"];
                    int id_Patient = (int)dr["id_Patient"];
                    bool came_to_rdv = (bool)dr["came_to_rdv"];
                    bool a_paye = (bool)dr["a_paye"];

                    DateTime dar = (DateTime)dr["DateRendezVous"];
                    DateTime hd = (DateTime)dr["HeureDepart"];
                    DateTime hf = (DateTime)dr["HeureFin"];
                    DateTime Debut = new DateTime(dar.Year, dar.Month, dar.Day, hd.Hour, hd.Minute, hd.Second, 0);
                    DateTime Fin = new DateTime(dar.Year, dar.Month, dar.Day, hf.Hour, hf.Minute, hf.Second, 0);
                    string Titre = "";

                    #region Selection du Titre de base
                    DataSet dds = new DataSet();
                    SqlManager.DbPath = vPathBdSource;
                    SqlManager.ExecuteSql("SELECT Nom, Prenom FROM Patient WHERE id_Patient = " + id_Patient + ";", ref dds);
                    Titre = (string)dds.Tables[0].Rows[0]["Nom"];
                    #endregion

                    string Message = "";

                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO RendezVous (id_Rdv, id_Patient, came_to_rdv,a_paye, Debut, Fin, "
                    + "Titre, Message)" + " VALUES(" + id_Rdv + "," + id_Patient + "," + came_to_rdv + ","
                    + a_paye + ",FORMAT(#" + Debut.ToString() + "#,\"dd/mm/yyyy hh:nn\"),FORMAT(#"
                    + Fin.ToString() + "#, \"dd/mm/yyyy hh:nn\"), \"" + Titre + "\",\"" + Message + "\");";

                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des Prescriptions
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des prescriptions... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Prescription;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des Prescriptions dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des photos dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Patient = (int)dr["id_Patient"];
                    int id_Medicament = (int)dr["id_Medicament"];
                    DateTime Date_P = (DateTime)dr["Date_P"];
                    string Rmq = (string)dr["Rmq"];
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Photo (id_Patient, id_Medicament, Date_P, Rmq) VALUES("
                        + id_Patient + "," + id_Medicament + "," + Date_P + ",\"" + Rmq + "\");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des Photos
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des photos... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Photo;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des Photos dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows * 5, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des photos dans la nouvelle base de donnes...");

                try
                {
                    MemoryStream ms;
                    Image bmp;
                    foreach (DataRow dr in ds.Tables[0].Rows)
                    {
                        #region Variables
                        int id_Photo = (int)dr["id_Photo"];
                        int id_Patient = (int)dr["id_Patient"];
                        string Path = (string)dr["Path"];

                        string Rmq = "";
                        if (!dr["Remarques"].Equals(DBNull.Value))
                            Rmq = (string)dr["Remarques"];
                        #endregion

                        ms = new MemoryStream();
                        SqlManager.PrepareProcedure("INSERT_Photo", false);
                        SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = id_Patient;//pat.IDPatient;
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        bmp = new Bitmap(Path);
                        bmp = new Bitmap(bmp, new Size(bmp.Width / 3, bmp.Height / 3));
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Pic]", OleDbType.Binary).Value = ms.ToArray();
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        bmp = new Bitmap(bmp, new Size(100, 100));
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Thumb]", OleDbType.Binary).Value = ms.ToArray();
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        SqlManager.Parameters.Add("[_Rmq]", OleDbType.Variant).Value = Rmq;
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        object LastId = 0;
                        SqlManager.ExecuteNonQueryProcedure(ref LastId);
                        SqlManager.Parameters.Clear();
                        frame.pb_ProgressBar.Invoke(this.PerformStep);
                    }
                    frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                }
                catch (Exception ex) { frame.tb_Log.Invoke(this.Log, " ERREUR: " + ex.Message + "\r\n"); }
                #endregion

                frame.tb_Log.Invoke(this.Log, "\r\nCREATION DE LA NOUVELLE BD TERMINEE...");
            }
            catch (Exception ex)
            {
                frame.tb_Log.Invoke(this.Log, " ERREUR (" + ex.Message + ")");
            }
        }

        #region GUI
        /// <summary>
        /// Fait une modification de la GUI (Met la ProgressBar en style Block et log dans le fentre de log)
        /// </summary>
        private void GuiLogAndProgressBar(string _ToLog, ProgressBarStyle _Style)
        {
            frame.tb_Log.Text += _ToLog;
            frame.pb_ProgressBar.Style = _Style;
        }
        /// <summary>
        /// Fait faire une tape la ProgressBar
        /// </summary>
        private void GuiPerformStep()
        {
            frame.pb_ProgressBar.PerformStep();
        }
        /// <summary>
        /// Log dans le TextBox
        /// </summary>
        /// <param name="_ToLog"></param>
        private void GuiLog(string _ToLog)
        {
            frame.tb_Log.Text += _ToLog;
        }
        /// <summary>
        /// Initialise la ProgressBar
        /// </summary>
        /// <param name="_Maximum">Minimum</param>
        /// <param name="_Minimum">Maximum</param>
        /// <param name="_Step">Valeur de saur</param>
        /// <param name="_Value">Valeur de l'tape</param>
        private void GuiSetupProgressBar(int _Maximum, int _Minimum, int _Step, int _Value)
        {
            //frame.ProgressBar
            frame.pb_ProgressBar.Value = _Value;
            frame.pb_ProgressBar.Maximum = _Maximum;
            frame.pb_ProgressBar.Minimum = _Minimum;
            frame.pb_ProgressBar.Step = _Step;
        }
        #endregion
        #endregion

        #region Accesseurs
        /// <summary>
        /// Emplacement de la Base de Donnes 1
        /// </summary>
        public string PathDB1 { get { return vPathBdSource; } set { vPathBdSource = value; } }
        /// <summary>
        /// Emplacement de la Base de Donnes 2
        /// </summary>
        public string PathDB2 { get { return vPathBdTarget; } set { vPathBdTarget = value; } }
        /// <summary>
        /// Retourne ou initialise la frame de la GUI
        /// </summary>
        public fr_Main GUI { get { return frame; } set { frame = value; } }
        #endregion
    }
    /// <summary>
    /// Thread for the version 0.9.13V1 database
    /// </summary>
    internal class ThreadProcessorV0913v1 : IThreadProcessor
    {
        #region variables
        /// <summary>
        /// Emplacement de la BD
        /// </summary>
        private string vPathBdSource = "";
        /// <summary>
        /// Path de la deuxime base de donnes
        /// </summary>
        private string vPathBdTarget = "";
        /// <summary>
        /// Frame de la GUI
        /// </summary>
        private fr_Main frame;
        /// <summary>
        /// Sql Manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.GetInstance().Path);

        #region PerformStep
        /// <summary>
        /// Delegate pour l'eccutio d'un pas  la ProgressBar
        /// </summary>
        private delegate void GuiPerformStepHandler();
        /// <summary>
        /// Mthode de traitement pour l'excution d'un pas a la ProgressBar
        /// </summary>
        private GuiPerformStepHandler PerformStep;
        #endregion

        #region Log
        /// <summary>
        /// Delegate pour le loggage dans la TextBox
        /// </summary>
        /// <param name="_ToLog">Texte  logger</param>
        private delegate void GuiLogHandler(string _ToLog);

        /// <summary>
        /// Mthode pour le loggage dans le Text Box
        /// </summary>
        private GuiLogHandler Log;
        #endregion

        #region Setup ProgressBar
        /// <summary>
        /// Dekegate pour le setup de la ProgressBar
        /// </summary>
        /// <param name="_Maximum">Maximum de la ProgresBar</param>
        /// <param name="_Minimum">Minimum de la ProgresBar</param>
        /// <param name="_Step">Nombre d'tapes  faire lors d'un pas</param>
        /// <param name="_Value">Valeur de base du pas de la ProgressBAr</param>
        private delegate void GuiSetupProgressBarHandler(int _Maximum, int _Minimum, int _Step, int _Value);

        /// <summary>
        /// Mthode pour le setup de la ProgressBar
        /// </summary>
        private GuiSetupProgressBarHandler SetupProgressBar;
        #endregion
        #endregion

        #region Constructeur
        #endregion

        #region Mthodes
        /// <summary>
        /// Process de transfert d'une base de donnes vers une autre
        /// </summary>
        public void Process()
        {
            try
            {

                #region Selection des types de mdicaments
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des types de mdicaments... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                DataSet ds = new DataSet();
                SqlManager.DbPath = vPathBdSource;
                int rows = SqlManager.ExecuteSql("SELECT * FROM Type_Medicament;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des types de mdicaments dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des mdicaments dans la nouvelle base de donnes...");

                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int ID_Type = (int)dr["ID_Type"];

                    string Rmq = "";
                    if (!dr["Rmq"].Equals(DBNull.Value))
                        Rmq = (string)dr["Rmq"];
                    Rmq = Rmq.Replace("\"", "");

                    string Type = "";
                    if (!dr["Type"].Equals(DBNull.Value))
                        Type = (string)dr["Type"];
                    Type = Type.Replace("\"", "");
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Type_Medicament (ID_Type, Type, Rmq) VALUES("
                        + ID_Type + ",\"" + Rmq + "\",\"" + Type + "\");";
                    #endregion

                    #region Execute Query

                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des Mdicaments
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des mdicaments... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Medicament;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des mdicaments dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des mdicaments dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Medicament = (int)dr["id_Medicament"];

                    int id_Type = -1;
                    if (!dr["id_Type"].Equals(DBNull.Value))
                        id_Type = (int)dr["id_Type"];

                    string Rmq = "";
                    if (!dr["Rmq"].Equals(DBNull.Value))
                        Rmq = (string)dr["Rmq"];
                    Rmq = Rmq.Replace("\"", "");

                    string Nom = "";
                    if (!dr["Nom"].Equals(DBNull.Value))
                        Nom = (string)dr["Nom"];
                    Nom = Nom.Replace("\"", "");
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Medicament (id_Medicament, id_Type, Rmq, Nom) VALUES("
                        + id_Medicament + "," + id_Type + ",\"" + Rmq + "\",\"" + Nom + "\");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des rputations
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des rputations... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Reputation;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des rputations dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des rputations dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Reputation = (int)dr["id_Reputation"];

                    string Label = "";
                    if (!dr["Label"].Equals(DBNull.Value))
                        Label = (string)dr["Label"];
                    Label = Label.Replace("\"", "");

                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Reputation (id_Reputation, Label) VALUES("
                        + id_Reputation + ",\"" + Label + "\");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des honoraires

                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des honoraires... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Honoraire;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des honoraires dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des honoraires dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Honoraire = (int)dr["id_Honoraire"];

                    string Type = "";
                    if (!dr["Type"].Equals(DBNull.Value))
                        Type = (string)dr["Type"];
                    Type = Type.Replace("\"", "");

                    short Prix = 0;
                    if (!dr["Prix"].Equals(DBNull.Value))
                        Prix = (short)dr["Prix"];
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Honoraire (id_Honoraire, Type, Prix) VALUES("
                        + id_Honoraire + ",\"" + Type + "\"," + Prix + ");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des mdecins
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des mdecins... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Medecin;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des mdecins dans la nouvelle DB

                #region Create and process Sql Query
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des mdecins dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_medecin = (int)dr["id_medecin"];

                    string Nom = " ";
                    if (!dr["Nom"].Equals(DBNull.Value))
                        Nom = (string)dr["Nom"];
                    Nom = Nom.Replace("\"", "");

                    string Prenom = " ";
                    if (!dr["Prenom"].Equals(DBNull.Value))
                        Prenom = (string)dr["Prenom"];
                    Prenom = Prenom.Replace("\"", "");

                    string Adresse = " ";
                    if (!dr["Adresse"].Equals(DBNull.Value))
                        Adresse = (string)dr["Adresse"];
                    Adresse = Adresse.Replace("\"", "");

                    string Ville = " ";
                    if (!dr["Ville"].Equals(DBNull.Value))
                        Ville = (string)dr["Ville"];
                    Ville = Ville.Replace("\"", "");

                    string CodePostal = " ";
                    if (!dr["CodePostal"].Equals(DBNull.Value))
                        CodePostal = (string)dr["CodePostal"];
                    CodePostal = CodePostal.Replace("\"", "");

                    string Telephone = " ";
                    if (!dr["Telephone"].Equals(DBNull.Value))
                        Telephone = (string)dr["Telephone"];
                    Telephone = Telephone.Replace("\"", "");

                    string Fax = " ";
                    if (!dr["Fax"].Equals(DBNull.Value))
                        Fax = (string)dr["Fax"];
                    Fax = Fax.Replace("\"", "");

                    bool Homeopathe = true;
                    if (!dr["Homeopathe"].Equals(DBNull.Value))
                        Homeopathe = (bool)dr["Homeopathe"];

                    bool MedecinTraitant = true;
                    if (!dr["MedecinTraitant"].Equals(DBNull.Value))
                        MedecinTraitant = (bool)dr["MedecinTraitant"];
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Medecin (id_medecin, Nom, Prenom, Adresse, Ville, CodePostal, "
                        + "Telephone, Fax, Homeopathe, MedecinTraitant) VALUES(" + id_medecin + ",\"" + Nom + "\",\""
                        + Prenom + "\",\"" + Adresse + "\",\"" + Ville + "\",\"" + CodePostal + "\",\"" + Telephone
                        + "\",\"" + Fax + "\"," + Homeopathe + "," + MedecinTraitant + ")";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #endregion

                #region Selection des patients
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des patients... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM PATIENT;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des patient dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des patients dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Patient = (int)dr["id_Patient"];

                    string Nom = "";
                    if (!dr["Nom"].Equals(DBNull.Value))
                        Nom = (string)dr["Nom"];
                    Nom = Nom.Replace("\"", "");

                    string Prenom = "";
                    if (!dr["Prenom"].Equals(DBNull.Value))
                        Prenom = (string)dr["Prenom"];
                    Prenom = Prenom.Replace("\"", "");

                    string LieuNaissance = "";
                    if (!dr["Lieu_Naissance"].Equals(DBNull.Value))
                        LieuNaissance = (string)dr["Lieu_Naissance"];
                    LieuNaissance = LieuNaissance.Replace("\"", "");

                    DateTime DateNaissance = new DateTime();
                    if (!dr["DateNaissance"].Equals(DBNull.Value))
                        DateNaissance = (DateTime)dr["DateNaissance"];

                    string Adresse = "";
                    if (!dr["Adresse"].Equals(DBNull.Value))
                        Adresse = (string)dr["Adresse"];
                    Adresse = Adresse.Replace("\"", "");

                    string Code_Postal = "";
                    if (!dr["Code_Postal"].Equals(DBNull.Value))
                        Code_Postal = (string)dr["Code_Postal"];
                    Code_Postal = Code_Postal.Replace("\"", "");

                    string Profession = "";
                    if (!dr["Profession"].Equals(DBNull.Value))
                        Profession = (string)dr["Profession"];
                    Profession = Profession.Replace("\"", "");

                    string Ville = "";
                    if (!dr["Ville"].Equals(DBNull.Value))
                        Ville = (string)dr["Ville"];
                    Ville = Ville.Replace("\"", "");

                    string TelPriv = "";
                    if (!dr["Telephone_Prive"].Equals(DBNull.Value))
                        TelPriv = (string)dr["Telephone_Prive"];
                    TelPriv = TelPriv.Replace("\"", "");

                    string TelPro = "";
                    if (!dr["Telephone_Professionel"].Equals(DBNull.Value))
                        TelPro = (string)dr["Telephone_Professionel"];
                    TelPro = TelPro.Replace("\"", "");

                    string Sexe = "";
                    if (!dr["Sexe"].Equals(DBNull.Value))
                        Sexe = (string)dr["Sexe"];
                    Sexe = Sexe.Replace("\"", "");

                    string Mail = "";
                    if (!dr["Mail"].Equals(DBNull.Value))
                        Mail = (string)dr["Mail"];
                    Mail = Mail.Replace("\"", "");

                    int idr = -1;
                    if (!dr["id_Reputation"].Equals(DBNull.Value))
                        idr = (int)dr["id_Reputation"];

                    int idh = -1;
                    if (!dr["id_Homeopathe"].Equals(DBNull.Value))
                        idh = (int)dr["id_Homeopathe"];

                    int idm = -1;
                    if (!dr["id_Medecin"].Equals(DBNull.Value))
                        idm = (int)dr["id_Medecin"];

                    int idho = -1;
                    try
                    {

                        if (!dr["id_honoraire"].Equals(DBNull.Value))
                            idho = (int)dr["id_honoraire"];
                    }
                    catch (Exception) { idho = -1; }

                    string Motif1 = "";
                    if (!dr["Motif_1"].Equals(DBNull.Value))
                        Motif1 = (string)dr["Motif_1"];
                    Motif1 = Motif1.Replace("\"", "");

                    string Motif2 = "";
                    if (!dr["Motif_2"].Equals(DBNull.Value))
                        Motif2 = (string)dr["Motif_2"];
                    Motif2 = Motif2.Replace("\"", "");

                    DateTime Derniere_MAJ = new DateTime();
                    if (!dr["Derniere_MAJ"].Equals(DBNull.Value))
                        Derniere_MAJ = (DateTime)dr["Derniere_MAJ"];

                    string FicheMedicale = "";
                    if (!dr["FicheMedicale"].Equals(DBNull.Value))
                        FicheMedicale = (string)dr["FicheMedicale"];
                    FicheMedicale = FicheMedicale.Replace("\"", "");

                    string Biodynamics = "";
                    if (!dr["Biodynamics"].Equals(DBNull.Value))
                        Biodynamics = (string)dr["Biodynamics"];
                    Biodynamics = Biodynamics.Replace("\"", "");

                    string HE = "";
                    if (!dr["HE"].Equals(DBNull.Value))
                        HE = (string)dr["HE"];
                    HE = HE.Replace("\"", "");

                    string Kinergetique = "";
                    if (!dr["Kinergetique"].Equals(DBNull.Value))
                        Kinergetique = (string)dr["Kinergetique"];
                    Kinergetique = Kinergetique.Replace("\"", "");

                    string FICHE_SUPPL_1 = "";
                    if (!dr["FICHE_SUPPL_1"].Equals(DBNull.Value))
                        FICHE_SUPPL_1 = (string)dr["FICHE_SUPPL_1"];
                    FICHE_SUPPL_1 = FICHE_SUPPL_1.Replace("\"", "");

                    string FICHE_SUPPL_2 = "";
                    if (!dr["FICHE_SUPPL_2"].Equals(DBNull.Value))
                        FICHE_SUPPL_2 = (string)dr["FICHE_SUPPL_2"];
                    FICHE_SUPPL_2 = FICHE_SUPPL_2.Replace("\"", "");

                    string FICHE_SUPPL_3 = "";
                    if (!dr["FICHE_SUPPL_3"].Equals(DBNull.Value))
                        FICHE_SUPPL_3 = (string)dr["FICHE_SUPPL_3"];
                    FICHE_SUPPL_3 = FICHE_SUPPL_3.Replace("\"", "");

                    string FICHE_SUPPL_4 = "";
                    if (!dr["FICHE_SUPPL_4"].Equals(DBNull.Value))
                        FICHE_SUPPL_4 = (string)dr["FICHE_SUPPL_4"];
                    FICHE_SUPPL_4 = FICHE_SUPPL_4.Replace("\"", "");
                    string rdvRemarques = "";
                    if (!dr["rdvRemarques"].Equals(DBNull.Value))
                        rdvRemarques = (string)dr["rdvRemarques"];
                    rdvRemarques = rdvRemarques.Replace("\"", "");

                    bool Mauvais_Payeur = false;
                    if (!dr["Mauvais_Payeur"].Equals(DBNull.Value))
                        Mauvais_Payeur = (bool)dr["Mauvais_Payeur"];

                    DateTime Prem_Consult = DateTime.Today;
                    if (!dr["Prem_Consult"].Equals(DBNull.Value))
                        Prem_Consult = (DateTime)dr["Prem_Consult"];

                    int Dette_Patient = 0;
                    try
                    {
                        if (!dr["Dette_Patient"].Equals(DBNull.Value))
                            Dette_Patient = (int)dr["Dette_Patient"];
                    }
                    catch (Exception) { Dette_Patient = 0; }


                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Patient ( id_Patient,Nom, Prenom, Lieu_Naissance, DateNaissance,"
                        + " Adresse, Code_Postal, Profession, Ville, Telephone_Prive, Telephone_Professionel, Sexe, Mail, "
                        + "id_Reputation, id_Medecin,id_Homeopathe,id_honoraire,  Motif_1, Motif_2, Derniere_MAJ, "
                        + "FicheMedicale, Biodynamics, HE, Kinergetique, FICHE_SUPPL_1, FICHE_SUPPL_2, FICHE_SUPPL_3,"
                        + " FICHE_SUPPL_4, rdvRemarques, Mauvais_Payeur, Dette_Patient,Prem_Consult, Honoraire) "
                        + "VALUES (" + id_Patient + ",\"" + Nom + "\", \"" + Prenom + "\", \"" + LieuNaissance
                        + "\", FORMAT(#" + DateNaissance.ToShortDateString() + "#,\"dd/mm/yyyy\"), \"" + Adresse
                        + "\", \"" + Code_Postal + "\", \"" + Profession + "\", \"" + Ville + "\", \"" + TelPriv
                        + "\", \"" + TelPro + "\", \"" + Sexe + "\", \"" + Mail + "\", ";

                    if (idr == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idr + ",";

                    if (idh == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idh + ",";

                    if (idm == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idm + ",";

                    if (idho == -1)
                        SqlQuery += "NULL,";
                    else
                        SqlQuery += idho + ",";

                    SqlQuery += "\"" + Motif1 + "\", \"" + Motif2
                    + "\", FORMAT(#" + Derniere_MAJ.ToShortDateString() + "#,\"dd/mm/yyyy\"), \""
                    + FicheMedicale.Replace("\"", "") + "\",\""
                    + Biodynamics.Replace("\"", "").Replace("'", "") + "\",\"" + HE.Replace("\"", "").Replace("'", "") + "\",\""
                    + Kinergetique.Replace("\"", "").Replace("'", "") + "\",\""
                    + FICHE_SUPPL_1.Replace("\"", "").Replace("'", "") + "\",\""
                    + FICHE_SUPPL_2.Replace("\"", "").Replace("'", "") + "\",\""
                    + FICHE_SUPPL_3.Replace("\"", "").Replace("'", "") + "\",\""
                    + FICHE_SUPPL_4.Replace("\"", "").Replace("'", "") + "\",\""
                    + rdvRemarques.Replace("\"", "").Replace("'", "")
                    + "\"," + Mauvais_Payeur + "," + Dette_Patient + ",FORMAT(#"
                    + Prem_Consult + "#,\"dd/mm/yyyy\"),0);";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des rendez-vous
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des rendez-vous... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM RendezVous;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des rendez-vous dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des rendez-vous dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {

                    #region Variables
                    int id_Rdv = (int)dr["id_Rdv"];
                    int id_Patient = (int)dr["id_Patient"];
                    bool came_to_rdv = (bool)dr["came_to_rdv"];
                    bool a_paye = (bool)dr["a_paye"];

                    DateTime Debut = (DateTime)dr["Debut"];
                    DateTime Fin = (DateTime)dr["Fin"];

                    string Titre = (string)dr["Titre"];
                    Titre = Titre.Replace("\"", "");
                    string Message = (string)dr["Message"];
                    Message = Message.Replace("\"", "");

                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO RendezVous (id_Rdv, id_Patient, came_to_rdv,a_paye, Debut, Fin, "
                    + "Titre, Message)" + " VALUES(" + id_Rdv + "," + id_Patient + "," + came_to_rdv + ","
                    + a_paye + ",FORMAT(#" + Debut.ToString() + "#,\"dd/mm/yyyy hh:nn\"),FORMAT(#"
                    + Fin.ToString() + "#, \"dd/mm/yyyy hh:nn\"), \"" + Titre + "\",\"" + Message + "\");";

                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des Prescriptions
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des prescriptions... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Prescription;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des Prescriptions dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des prescriptions dans la nouvelle base de donnes...");
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    #region Variables
                    int id_Prescription = (int)dr["id_prescription"];
                    int id_Patient = (int)dr["id_Patient"];
                    int id_Medicament = (int)dr["id_Medicament"];
                    DateTime Date_P = (DateTime)dr["Date_P"];
                    string Rmq = (string)dr["Rmq"];
                    Rmq = Rmq.Replace("\"", "");
                    #endregion

                    #region SQL Query
                    string SqlQuery = "INSERT INTO Prescription (ID_Prescription, id_Patient, id_Medicament, "
                        + "Date_P, Rmq)" + " VALUES(" + id_Prescription + "," + id_Patient + ","
                        + id_Medicament + "," + "FORMAT(#" + Date_P.ToString() + "#, \"dd/mm/yyyy hh:nn\")"
                        + ",\"" + Rmq + "\");";
                    #endregion

                    #region Execute Query
                    SqlManager.DbPath = vPathBdTarget;
                    SqlManager.ExecuteNonQuery(SqlQuery);
                    frame.pb_ProgressBar.Invoke(this.PerformStep);
                    #endregion
                }
                frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                #endregion

                #region Selection des Photos
                #region GuiWork
                this.Log = new GuiLogHandler(this.GuiLog);
                frame.tb_Log.Invoke(Log, "Selection des photos... ");

                this.SetupProgressBar = new GuiSetupProgressBarHandler(this.GuiSetupProgressBar);
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { 2, 0, 1, 0 });

                this.PerformStep = new GuiPerformStepHandler(this.GuiPerformStep);
                frame.pb_ProgressBar.Invoke(PerformStep);
                #endregion

                ds.Clear();
                SqlManager.DbPath = vPathBdSource;
                rows = SqlManager.ExecuteSql("SELECT * FROM Photo;", ref ds);

                #region Gui Work
                frame.pb_ProgressBar.Invoke(PerformStep);
                frame.tb_Log.Invoke(Log, " " + rows + " entes.\r\n");
                frame.pb_ProgressBar.Invoke(SetupProgressBar, new object[] { rows, 0, 0, 0 });
                #endregion
                #endregion

                #region Ajout des Photos dans la nouvelle BD
                frame.pb_ProgressBar.Invoke(this.SetupProgressBar, new object[] { rows * 5, 0, 1, 0 });
                frame.tb_Log.Invoke(this.Log, "Ajout des photos dans la nouvelle base de donnes...");

                try
                {
                    MemoryStream ms;                    
                    foreach (DataRow dr in ds.Tables[0].Rows)
                    {
                        #region Variables
                        int id_Photo = (int)dr["id_Photo"];
                        int id_Patient = (int)dr["id_Patient"];

                        string Rmq = "";
                        if (!dr["Remarques"].Equals(DBNull.Value))
                            Rmq = (string)dr["Remarques"];
                        Rmq = Rmq.Replace("\"", "");
                        #endregion

                        ms = new MemoryStream((byte[])dr["BinPic"]);
                        SqlManager.PrepareProcedure("INSERT_Photo");
                        SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = id_Patient;//pat.IDPatient;
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        SqlManager.Parameters.Add("[_Pic]", OleDbType.Binary).Value = ms.ToArray();
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        ms = new MemoryStream((byte[])dr["Thumb"]);
                        SqlManager.Parameters.Add("[_Thumb]", OleDbType.Binary).Value = ms.ToArray();
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        SqlManager.Parameters.Add("[_Rmq]", OleDbType.Variant).Value = Rmq;
                        frame.pb_ProgressBar.Invoke(this.PerformStep);

                        object LastId = 0;
                        SqlManager.ExecuteNonQueryProcedure(ref LastId);
                        SqlManager.Parameters.Clear();
                        frame.pb_ProgressBar.Invoke(this.PerformStep);
                    }
                    frame.tb_Log.Invoke(this.Log, " OK\r\n\r\n");
                }
                catch (Exception ex) { frame.tb_Log.Invoke(this.Log, " ERREUR: " + ex.Message + "\r\n"); }
                #endregion


                frame.tb_Log.Invoke(this.Log, "\r\nCREATION DE LA NOUVELLE BD TERMINEE...");
            }
            catch (Exception ex)
            {
                frame.tb_Log.Invoke(this.Log, " ERREUR (" + ex.Message + ")");
            }
        }

        #region GUI
        /// <summary>
        /// Fait une modification de la GUI (Met la ProgressBar en style Block et log dans le fentre de log)
        /// </summary>
        private void GuiLogAndProgressBar(string _ToLog, ProgressBarStyle _Style)
        {
            frame.tb_Log.Text += _ToLog;
            frame.pb_ProgressBar.Style = _Style;
        }

        /// <summary>
        /// Fait faire une tape la ProgressBar
        /// </summary>
        private void GuiPerformStep()
        {
            frame.pb_ProgressBar.PerformStep();
        }

        /// <summary>
        /// Log dans le TextBox
        /// </summary>
        /// <param name="_ToLog"></param>
        private void GuiLog(string _ToLog)
        {
            frame.tb_Log.Text += _ToLog;
        }

        /// <summary>
        /// Initialise la ProgressBar
        /// </summary>
        /// <param name="_Maximum">Minimum</param>
        /// <param name="_Minimum">Maximum</param>
        /// <param name="_Step">Valeur de saur</param>
        /// <param name="_Value">Valeur de l'tape</param>
        private void GuiSetupProgressBar(int _Maximum, int _Minimum, int _Step, int _Value)
        {
            //frame.ProgressBar
            frame.pb_ProgressBar.Value = _Value;
            frame.pb_ProgressBar.Maximum = _Maximum;
            frame.pb_ProgressBar.Minimum = _Minimum;
            frame.pb_ProgressBar.Step = _Step;
        }
        #endregion
        #endregion

        #region Accesseurs
        /// <summary>
        /// Emplacement de la Base de Donnes 1
        /// </summary>
        public string PathDB1 { get { return vPathBdSource; } set { vPathBdSource = value; } }
        /// <summary>
        /// Emplacement de la Base de Donnes 2
        /// </summary>
        public string PathDB2 { get { return vPathBdTarget; } set { vPathBdTarget = value; } }
        /// <summary>
        /// Retourne ou initialise la frame de la GUI
        /// </summary>
        public fr_Main GUI { get { return frame; } set { frame = value; } }
        #endregion
    }
}